const pages = require('../../mixin/pages')
const { getProductCategoryList } = require('../../api/common')
const { getLunboList } = require('./api/index')
let leftHeight = 0
let rightHeight = 0
let query = null

Page({

  /**
   * 页面的初始数据
   */
  data: {
    ...pages.data(),
    listUrl: '/api/user/home/product/list',
    searchForm: {
      key_words: '',
      category_id: ''
    },
    txtlist: [],
    headerNav: [
      {
        icon: 'partner@2x.png',
        name: '供应商',
        path: 'business'
      },
      {
        icon: 'guide@2x.png',
        name: '采购指南',
        path: 'guide'
      },
      {
        icon: 'collection@2x.png',
        name: '收藏',
        path: 'collection'
      },
      {
        icon: 'news.png',
        name: '农事天地',
        path: 'news'
      }
    ],
    categoryList: [],
    leftList: [],
    rightList: []
  },
  ...pages.methods,
  /**
   * 生命周期函数--监听页面加载
   */
  async onLoad(options) {
    this.fetchLunboList()
    await this.fetchProductCategoryList()
    this.fetchOrderList()
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {
    this.getTabBar().init()
  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {
    if (this.data.freshing) {
      return
    }
    this.setData({
      freshing: true
    })
    this.bindCallBack()
  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {
    this.fetchOrderList()
  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },
  bindCallBack() {
    this.refreshOrderList()
    this.fetchLunboList()
  },
  async fetchLunboList() {
    try {
      const { status, data, msg } = await getLunboList()
      if (status) {
        const { lunbos } = data
        this.setData({
          lunbos: lunbos
        })
      } else {
        wx.showToast({
          title: msg,
          icon: 'none'
        })
      }
    } catch (err) {}
  },
  async fetchProductCategoryList() {
    try {
      const { status, data, msg } = await getProductCategoryList()
      if (status && Array.isArray(data) && data.length > 0) {
        this.setData({
          categoryList: data.map(item => {
            return {
              ...item,
              text: item.category_name
            }
          }),
          'searchForm.category_id': data[0].id
        })
      } else {
        wx.showToast({
          title: msg,
          icon: 'none'
        })
      }
    } catch (err) {}
  },
  handleTab(event) {
    const { name } = event.detail

    this.setData({
      'searchForm.category_id': name
    }, () => {
      this.refreshOrderList()
    })
  },
  async isLeft(list) {
    const {
      leftList,
      rightList
    } = this.data
    query = wx.createSelectorQuery()
    for (const item of list) {
      leftHeight <= rightHeight ? leftList.push(item) : rightList.push(item)
      await this.getBoxHeight(leftList, rightList)
    }
  },
  getBoxHeight(leftList, rightList) {
    return new Promise((resolve, reject) => {
      this.setData({
        leftList,
        rightList
      }, () => {
        query.select('.waterfall-left').boundingClientRect()
        query.select('.waterfall-right').boundingClientRect()
        query.exec((res) => {
          leftHeight = res[0].height
          rightHeight = res[1].height
          resolve()
        })
      })
    })
  },
  jumpLeavingAMessage() {
    wx.navigateTo({
      url: '/pages/leavingAMessage/leavingAMessage'
    })
  },
  handleNav(e) {
    const { item } = e.currentTarget.dataset
    if (Object.prototype.toString.call(item) === '[object Object]' && item.path) {
      const path = item.path

      if (path === 'news' || path === 'partner') {
        wx.switchTab({
          url: `/pages/${path}/${path}`
        })
        return
      }
      wx.navigateTo({
        url: `/pages/${path}/${path}`
      })
    }
  },
  openMarketing(e) {
    const { page } = e.currentTarget.dataset

    wx.navigateTo({
      url: `/pages/${page}/${page}`
    })
  }
})
